@@ -242,103 +242,86 @@
 
 /*-- Information Element Types --------------------*/
 /* prototype structure, all IEs start with these members */
-__WLAN_PRAGMA_PACK1__
+
 typedef struct wlan_ie
 {
-	UINT8	eid		__WLAN_ATTRIB_PACK__;
-	UINT8	len		__WLAN_ATTRIB_PACK__;
-} wlan_ie_t;
-__WLAN_PRAGMA_PACKDFLT__
+	UINT8	eid;
+	UINT8	len;
+} __WLAN_ATTRIB_PACK__ wlan_ie_t;
 
 /*-- Service Set Identity (SSID)  -----------------*/
-__WLAN_PRAGMA_PACK1__
 typedef struct wlan_ie_ssid
 {
-	UINT8	eid		__WLAN_ATTRIB_PACK__;
-	UINT8	len		__WLAN_ATTRIB_PACK__;
-	UINT8	ssid[1]		__WLAN_ATTRIB_PACK__;  /* may be zero, ptrs may overlap */
-} wlan_ie_ssid_t;
-__WLAN_PRAGMA_PACKDFLT__
+	UINT8	eid;
+	UINT8	len;
+	UINT8	ssid[1];  /* may be zero, ptrs may overlap */
+} __WLAN_ATTRIB_PACK__ wlan_ie_ssid_t;
 
 /*-- Supported Rates  -----------------------------*/
-__WLAN_PRAGMA_PACK1__
 typedef struct wlan_ie_supp_rates
 {
-	UINT8	eid		__WLAN_ATTRIB_PACK__;
-	UINT8	len		__WLAN_ATTRIB_PACK__;
-	UINT8	rates[1]	__WLAN_ATTRIB_PACK__;  /* had better be at LEAST one! */
-} wlan_ie_supp_rates_t;
-__WLAN_PRAGMA_PACKDFLT__
+	UINT8	eid;
+	UINT8	len;
+	UINT8	rates[1];  /* had better be at LEAST one! */
+} __WLAN_ATTRIB_PACK__ wlan_ie_supp_rates_t;
 
 /*-- FH Parameter Set  ----------------------------*/
-__WLAN_PRAGMA_PACK1__
 typedef struct wlan_ie_fh_parms
 {
-	UINT8	eid		__WLAN_ATTRIB_PACK__;
-	UINT8	len		__WLAN_ATTRIB_PACK__;
-	UINT16	dwell		__WLAN_ATTRIB_PACK__;
-	UINT8	hopset		__WLAN_ATTRIB_PACK__;
-	UINT8	hoppattern	__WLAN_ATTRIB_PACK__;
-	UINT8	hopindex	__WLAN_ATTRIB_PACK__;
-} wlan_ie_fh_parms_t;
-__WLAN_PRAGMA_PACKDFLT__
+	UINT8	eid;
+	UINT8	len;
+	UINT16	dwell;
+	UINT8	hopset;
+	UINT8	hoppattern;
+	UINT8	hopindex;
+} __WLAN_ATTRIB_PACK__ wlan_ie_fh_parms_t;
 
 /*-- DS Parameter Set  ----------------------------*/
-__WLAN_PRAGMA_PACK1__
 typedef struct wlan_ie_ds_parms
 {
-	UINT8	eid		__WLAN_ATTRIB_PACK__;
-	UINT8	len		__WLAN_ATTRIB_PACK__;
-	UINT8	curr_ch		__WLAN_ATTRIB_PACK__;
-} wlan_ie_ds_parms_t;
-__WLAN_PRAGMA_PACKDFLT__
+	UINT8	eid;
+	UINT8	len;
+	UINT8	curr_ch;
+} __WLAN_ATTRIB_PACK__ wlan_ie_ds_parms_t;
 
 /*-- CF Parameter Set  ----------------------------*/
-__WLAN_PRAGMA_PACK1__
+
 typedef struct wlan_ie_cf_parms
 {
-	UINT8	eid		__WLAN_ATTRIB_PACK__;
-	UINT8	len		__WLAN_ATTRIB_PACK__;
-	UINT8	cfp_cnt		__WLAN_ATTRIB_PACK__;
-	UINT8	cfp_period	__WLAN_ATTRIB_PACK__;
-	UINT16	cfp_maxdur	__WLAN_ATTRIB_PACK__;
-	UINT16	cfp_durremaining	__WLAN_ATTRIB_PACK__;
-} wlan_ie_cf_parms_t;
-__WLAN_PRAGMA_PACKDFLT__
+	UINT8	eid;
+	UINT8	len;
+	UINT8	cfp_cnt;
+	UINT8	cfp_period;
+	UINT16	cfp_maxdur;
+	UINT16	cfp_durremaining;
+} __WLAN_ATTRIB_PACK__ wlan_ie_cf_parms_t;
 
 /*-- TIM ------------------------------------------*/
-__WLAN_PRAGMA_PACK1__
 typedef struct wlan_ie_tim
 {
-	UINT8	eid		__WLAN_ATTRIB_PACK__;
-	UINT8	len		__WLAN_ATTRIB_PACK__;
-	UINT8	dtim_cnt	__WLAN_ATTRIB_PACK__;
-	UINT8	dtim_period	__WLAN_ATTRIB_PACK__;
-	UINT8	bitmap_ctl	__WLAN_ATTRIB_PACK__;
-	UINT8	virt_bm[1]	__WLAN_ATTRIB_PACK__;
-} wlan_ie_tim_t;
-__WLAN_PRAGMA_PACKDFLT__
+	UINT8	eid;
+	UINT8	len;
+	UINT8	dtim_cnt;
+	UINT8	dtim_period;
+	UINT8	bitmap_ctl;
+	UINT8	virt_bm[1];
+} __WLAN_ATTRIB_PACK__ wlan_ie_tim_t;
 
 /*-- IBSS Parameter Set ---------------------------*/
-__WLAN_PRAGMA_PACK1__
 typedef struct wlan_ie_ibss_parms
 {
-	UINT8	eid		__WLAN_ATTRIB_PACK__;
-	UINT8	len		__WLAN_ATTRIB_PACK__;
-	UINT16	atim_win	__WLAN_ATTRIB_PACK__;
-} wlan_ie_ibss_parms_t;
-__WLAN_PRAGMA_PACKDFLT__
+	UINT8	eid;
+	UINT8	len;
+	UINT16	atim_win;
+} __WLAN_ATTRIB_PACK__ wlan_ie_ibss_parms_t;
 
 /*-- Challenge Text  ------------------------------*/
-__WLAN_PRAGMA_PACK1__
 typedef struct wlan_ie_challenge
 {
-	UINT8	eid		__WLAN_ATTRIB_PACK__;
-	UINT8	len		__WLAN_ATTRIB_PACK__;
-	UINT8	challenge[1]	__WLAN_ATTRIB_PACK__;
-} wlan_ie_challenge_t;
-__WLAN_PRAGMA_PACKDFLT__
-
+	UINT8	eid;
+	UINT8	len;
+	UINT8	challenge[1];
+} __WLAN_ATTRIB_PACK__ wlan_ie_challenge_t;
 
 /*-------------------------------------------------*/
 /*  Frame Types  */
