@@ -148,7 +148,6 @@ typedef struct p80211_frmrx_t
 	UINT32  decrypt_err;
 } p80211_frmrx_t;
 
-
 #ifdef WIRELESS_EXT
 /* called by /proc/net/wireless */
 struct iw_statistics* p80211wext_get_wireless_stats(netdevice_t *dev);
@@ -157,6 +156,9 @@ int p80211wext_support_ioctl(netdevice_t *dev, struct ifreq *ifr, int cmd);
 #if WIRELESS_EXT > 12
 extern struct iw_handler_def p80211wext_handler_def;
 #endif
+
+int p80211wext_event_associated(struct wlandevice *wlandev, int assoc);
+
 #endif /* wireless extensions */
 
 /* WEP stuff */
@@ -213,7 +215,8 @@ typedef struct wlandevice
 	UINT8		bssid[WLAN_BSSID_LEN];
 	p80211pstr32_t	ssid;
 	UINT32		macmode;
-	UINT            shortpreamble;  /* C bool */
+	int             linkstatus;
+	int             shortpreamble;  /* C bool */
 
 	/* WEP State */
 	UINT8 wep_keys[NUM_WEPKEYS][MAX_KEYLEN];
@@ -221,7 +224,7 @@ typedef struct wlandevice
 	int   hostwep;
   
 	/* Request/Confirm i/f state (used by p80211) */
-	long			request_pending; /* flag, access atomically */
+	unsigned long		request_pending; /* flag, access atomically */
 
 	/* netlink socket */
 	/* queue for indications waiting for cmd completion */
@@ -254,6 +257,7 @@ typedef struct wlandevice
 
 #endif
 
+	struct mutex ioctl_lock;
 } wlandevice_t;
 
 /* WEP stuff */
@@ -278,6 +282,8 @@ void	p80211netdev_hwremoved(wlandevice_t *wlandev);
 void    p80211_suspend(wlandevice_t *wlandev);
 void    p80211_resume(wlandevice_t *wlandev);
 
+void p80211_allow_ioctls(wlandevice_t *wlandev);
+
 /*================================================================*/
 /* Function Definitions */
 
