@@ -48,14 +48,6 @@
 #ifndef _LINUX_P80211REQ_H
 #define _LINUX_P80211REQ_H
 
-#ifndef CONFIG_NETLINK
-#ifdef CONFIG_NETLINK_DEV
-#define CONFIG_NETLINK 1
-#elif CONFIG_NETLINK_DEV_MODULE
-#define CONFIG_NETLINK 1
-#endif
-#endif
-
 /*================================================================*/
 /* Constants */
 
@@ -72,13 +64,5 @@
 /* Function Declarations */
 
 int	p80211req_dorequest(wlandevice_t *wlandev, UINT8 *msgbuf);
-void	p80211req_confirm(wlandevice_t *wlandev);
-#ifdef CONFIG_NETLINK
-void	p80211indicate( wlandevice_t *wlandev, p80211msg_t *msg, int len);
-void	p80211indicate_init(void);
-void	p80211indicate_shutdown(void);
-void	p80211ind_mlme( wlandevice_t *wlandev, struct sk_buff *skb);
-void	p80211ind_distribution( wlandevice_t *wlandev, struct sk_buff *skb);
-#endif /* CONFIG_NETLINK */
 
 #endif
