@@ -89,6 +89,11 @@
 #endif
 
 #if defined(__KERNEL__)
+
+#ifndef AUTOCONF_INCLUDED
+#include <linux/config.h>
+#endif
+
 #if defined(__x86_64__)
 	#define WLAN_CPU_FAMILY		WLAN_x86_64
 	#define WLAN_SYSARCH		WLAN_PCAT
@@ -143,11 +148,13 @@
    as new ones are discovered, add them below. 
 */
 
+#if defined(WLAN_HOSTIF)
 #if ((WLAN_HOSTIF == WLAN_PCI) || (WLAN_HOSTIF == WLAN_PLX))
 #if ((WLAN_SYSARCH == WLAN_SKIFF) || (WLAN_SYSARCH == WLAN_PMAC) || (WLAN_SYSARCH == WLAN_SPARC))
 #define REVERSE_ENDIAN
 #endif
 #endif
+#endif
 
 /*=============================================================*/
 /*------ Bit settings -----------------------------------------*/
@@ -274,6 +281,16 @@ typedef int64_t		INT64;
 
 #if defined(__KERNEL__)
 
+#if ((LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)) || (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,19)))
+#define URB_ONLY_CALLBACK
+#endif
+
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19))
+#define PT_REGS    , struct pt_regs *regs
+#else
+#define PT_REGS
+#endif
+
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,7))
 #  define del_singleshot_timer_sync(a)  del_timer_sync(a)
 #endif
@@ -411,6 +428,10 @@ static inline void list_splice_init(struct list_head *list,
 #endif  // LIST_H
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
+#define SET_MODULE_OWNER(x)  do { } while (0)
+#endif
+
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,1,90))
 #define spin_lock(l)            do { } while (0)
 #define spin_unlock(l)          do { } while (0)
@@ -439,6 +460,13 @@ typedef int spinlock_t;
 #endif
 
 #ifdef _LINUX_PROC_FS_H
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
+#define PROC_NET  proc_net
+#else
+#define PROC_NET  init_net.proc_net
+#endif
+
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,25))
 
 extern inline struct proc_dir_entry *
@@ -462,6 +490,18 @@ create_proc_read_entry(const char *name, mode_t mode,
 #endif
 #endif /* _LINUX_PROC_FS_H */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,22)
+#define skb_reset_mac_header(__a)  (__a)->mac.raw = (__a)->data
+#define SKB_MAC_HEADER(__a) (__a)->mac.raw
+#else
+#define SKB_MAC_HEADER(__a) (__a)->mac_header
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
+#define IRQF_DISABLED SA_INTERRUPT
+#define IRQF_SHARED   SA_SHIRQ
+#endif
+
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0))
 #ifndef INIT_TQUEUE
 #define PREPARE_TQUEUE(_tq, _routine, _data)                    \
@@ -477,6 +517,12 @@ create_proc_read_entry(const char *name, mode_t mode,
         } while (0)
 #endif
 
+#ifndef container_of
+#define container_of(ptr, type, member) ({			\
+        const typeof( ((type *)0)->member ) *__mptr = (ptr);	\
+        (type *)( (char *)__mptr - offsetof(type,member) );})
+#endif
+
 #ifndef INIT_WORK
 #define work_struct tq_struct
 
@@ -487,10 +533,18 @@ create_proc_read_entry(const char *name, mode_t mode,
 #endif
 
 #define flush_scheduled_work  flush_scheduled_tasks
-#define INIT_WORK(_wq, _routine, _data)  INIT_TQUEUE(_wq, _routine, _data)
-#define PREPARE_WORK(_wq, _routine, _data)  PREPARE_TQUEUE(_wq, _routine, _data)
+#define INIT_WORK2(_wq, _routine)  INIT_TQUEUE(_wq, (void (*)(void *))_routine, _wq)
 #endif
-#endif // < 2.5 kernel
+
+#else // >= 2.5 kernel
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20)
+#define INIT_WORK2(_wq, _routine)	INIT_WORK(_wq, (void (*)(void *))_routine, _wq)
+#else
+#define INIT_WORK2(_wq, _routine)	INIT_WORK(_wq, _routine)
+#endif
+
+#endif // >= 2.5 kernel
 
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,38))
 typedef struct device netdevice_t;
@@ -650,6 +704,10 @@ typedef u32 pm_message_t;
         CardServices(ReportError, handle, err)
 #endif
 
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20))
+#define round_jiffies(a)  (a)
+#endif
+
 #endif /* __KERNEL__ */
 
 /*=============================================================*/
