@@ -54,7 +54,7 @@
 
 #define __NO_VERSION__		/* prevent the static definition */
 
-#include <linux/config.h>
+
 #include <linux/version.h>
 
 #include <linux/module.h>
@@ -258,7 +258,8 @@ int skb_ether_to_p80211( wlandevice_t *wlandev, UINT32 ethconv, struct sk_buff *
 }
 
 /* jkriegl: from orinoco, modified */
-void orinoco_spy_gather(wlandevice_t *wlandev, char *mac, p80211_rxmeta_t *rxmeta)
+static void orinoco_spy_gather(wlandevice_t *wlandev, char *mac,
+			       p80211_rxmeta_t *rxmeta)
 {
         int i;
 
@@ -269,8 +270,8 @@ void orinoco_spy_gather(wlandevice_t *wlandev, char *mac, p80211_rxmeta_t *rxmet
 
                 if (!memcmp(wlandev->spy_address[i], mac, ETH_ALEN)) {
 			memcpy(wlandev->spy_address[i], mac, ETH_ALEN);
-                        wlandev->spy_stat[i].level = rxmeta->signal - 0x95;
-                        wlandev->spy_stat[i].noise = rxmeta->noise - 0x95;
+                        wlandev->spy_stat[i].level = rxmeta->signal;
+                        wlandev->spy_stat[i].noise = rxmeta->noise;
                         wlandev->spy_stat[i].qual = (rxmeta->signal > rxmeta->noise) ? \
                                                      (rxmeta->signal - rxmeta->noise) : 0;
                         wlandev->spy_stat[i].updated = 0x7;
@@ -496,8 +497,16 @@ int skb_p80211_to_ether( wlandevice_t *wlandev, UINT32 ethconv, struct sk_buff *
 
 	}
 
+        /*
+         * Note that eth_type_trans() expects an skb w/ skb->data pointing
+         * at the MAC header, it then sets the following skb members:
+         * skb->mac_header, 
+         * skb->data, and
+         * skb->pkt_type.
+         * It then _returns_ the value that _we're_ supposed to stuff in
+         * skb->protocol.  This is nuts.
+         */
 	skb->protocol = eth_type_trans(skb, netdev);
-	skb->mac.raw = (unsigned char *) e_hdr; /* new MAC header */
 
         /* jkriegl: process signal and noise as set in hfa384x_int_rx() */
 	/* jkriegl: only process signal/noise if requested by iwspy */
