@@ -53,7 +53,7 @@
 /*================================================================*/
 /* System Includes */
 
-#include <linux/config.h>
+
 #include <linux/version.h>
 
 #include <linux/module.h>
@@ -167,11 +167,11 @@ void p80211netdev_startup(void)
 	DBFENTER;
 
 #ifdef CONFIG_PROC_FS
-	if (proc_net != NULL) {
+	if (PROC_NET != NULL) {
 		proc_p80211 = create_proc_entry(
 				"p80211", 
 				(S_IFDIR|S_IRUGO|S_IXUGO),
-				proc_net);
+				PROC_NET);
 	}
 #endif
 	DBFEXIT;
@@ -196,7 +196,7 @@ p80211netdev_shutdown(void)
 	DBFENTER;
 #ifdef CONFIG_PROC_FS
 	if (proc_p80211 != NULL) {
-		remove_proc_entry("p80211", proc_net);
+		remove_proc_entry("p80211", PROC_NET);
 	}
 #endif
 	DBFEXIT;
@@ -214,7 +214,7 @@ p80211netdev_shutdown(void)
 * Returns: 
 *	nothing
 ----------------------------------------------------------------*/
-int p80211knetdev_init( netdevice_t *netdev)
+static int p80211knetdev_init( netdevice_t *netdev)
 {
 	DBFENTER;
 	/* Called in response to register_netdev */
@@ -240,7 +240,7 @@ int p80211knetdev_init( netdevice_t *netdev)
 * Returns: 
 *	the address of the statistics structure
 ----------------------------------------------------------------*/
-struct net_device_stats*
+static struct net_device_stats*
 p80211knetdev_get_stats(netdevice_t *netdev)
 {
 	wlandevice_t	*wlandev = (wlandevice_t*)netdev->priv;
@@ -268,7 +268,7 @@ p80211knetdev_get_stats(netdevice_t *netdev)
 * Returns: 
 *	zero on success, non-zero otherwise
 ----------------------------------------------------------------*/
-int p80211knetdev_open( netdevice_t *netdev )
+static int p80211knetdev_open( netdevice_t *netdev )
 {
 	int 		result = 0; /* success */
 	wlandevice_t	*wlandev = (wlandevice_t*)(netdev->priv);
@@ -311,7 +311,7 @@ int p80211knetdev_open( netdevice_t *netdev )
 * Returns: 
 *	zero on success, non-zero otherwise
 ----------------------------------------------------------------*/
-int p80211knetdev_stop( netdevice_t *netdev )
+static int p80211knetdev_stop( netdevice_t *netdev )
 {
 	int		result = 0;
 	wlandevice_t	*wlandev = (wlandevice_t*)(netdev->priv);
@@ -369,7 +369,7 @@ p80211netdev_rx(wlandevice_t *wlandev, struct sk_buff *skb )
 * Side effects:
 *	
 ----------------------------------------------------------------*/
-void p80211netdev_rx_bh(unsigned long arg)
+static void p80211netdev_rx_bh(unsigned long arg)
 {
 	wlandevice_t *wlandev = (wlandevice_t *) arg;
 	struct sk_buff *skb = NULL; 
@@ -389,7 +389,8 @@ void p80211netdev_rx_bh(unsigned long arg)
 
 				/* set up various data fields */
 				skb->dev = dev;
-				skb->mac.raw = skb->data ;
+				skb_reset_mac_header(skb);
+
 				skb->ip_summed = CHECKSUM_NONE;
 				skb->pkt_type = PACKET_OTHERHOST;
 				skb->protocol = htons(ETH_P_80211_RAW); 
@@ -455,7 +456,7 @@ void p80211netdev_rx_bh(unsigned long arg)
 * Returns: 
 *	zero on success, non-zero on failure.
 ----------------------------------------------------------------*/
-int p80211knetdev_hard_start_xmit( struct sk_buff *skb, netdevice_t *netdev)
+static int p80211knetdev_hard_start_xmit( struct sk_buff *skb, netdevice_t *netdev)
 {
 	int		result = 0;
 	int		txresult = -1;
@@ -600,7 +601,7 @@ int p80211knetdev_hard_start_xmit( struct sk_buff *skb, netdevice_t *netdev)
 * Returns: 
 *	nothing
 ----------------------------------------------------------------*/
-void p80211knetdev_set_multicast_list(netdevice_t *dev)
+static void p80211knetdev_set_multicast_list(netdevice_t *dev)
 {
 	wlandevice_t	*wlandev = (wlandevice_t*)dev->priv;
 
@@ -691,7 +692,7 @@ static int p80211netdev_ethtool(wlandevice_t *wlandev, void __user *useraddr)
 *	Process thread (ioctl caller).  TODO: SMP support may require
 *	locks.
 ----------------------------------------------------------------*/
-int p80211knetdev_do_ioctl(netdevice_t *dev, struct ifreq *ifr, int cmd)
+static int p80211knetdev_do_ioctl(netdevice_t *dev, struct ifreq *ifr, int cmd)
 {
 	int			result = 0;
 	p80211ioctl_req_t	*req = (p80211ioctl_req_t*)ifr;
@@ -701,6 +702,7 @@ int p80211knetdev_do_ioctl(netdevice_t *dev, struct ifreq *ifr, int cmd)
 
 	WLAN_LOG_DEBUG(2, "rx'd ioctl, cmd=%d, len=%d\n", cmd, req->len);
 
+	mutex_lock(&wlandev->ioctl_lock);
 #if WIRELESS_EXT < 13
 	/* Is this a wireless extensions ioctl? */
 	if ((cmd >= SIOCIWFIRST) && (cmd <= SIOCIWLAST)) {
@@ -750,6 +752,8 @@ int p80211knetdev_do_ioctl(netdevice_t *dev, struct ifreq *ifr, int cmd)
 		result = -ENOMEM;
 	}
 bail:
+	mutex_unlock(&wlandev->ioctl_lock);
+
 	DBFEXIT;
 
 	return result; /* If allocate,copyfrom or copyto fails, return errno */
@@ -780,7 +784,7 @@ bail:
 *
 * by: Collin R. Mulliner <collin@mulliner.org>
 ----------------------------------------------------------------*/
-int p80211knetdev_set_mac_address(netdevice_t *dev, void *addr)
+static int p80211knetdev_set_mac_address(netdevice_t *dev, void *addr)
 {
 	struct sockaddr			*new_addr = addr;
 	p80211msg_dot11req_mibset_t	dot11req;
@@ -850,7 +854,7 @@ int p80211knetdev_set_mac_address(netdevice_t *dev, void *addr)
 	return result;
 }
 
-int wlan_change_mtu(netdevice_t *dev, int new_mtu)
+static int wlan_change_mtu(netdevice_t *dev, int new_mtu)
 {
 	DBFENTER;
 	// 2312 is max 802.11 payload, 20 is overhead, (ether + llc +snap)
@@ -919,6 +923,11 @@ int wlan_setup(wlandevice_t *wlandev)
 		dev->priv = wlandev;
 		dev->hard_start_xmit =	p80211knetdev_hard_start_xmit;
 		dev->get_stats =	p80211knetdev_get_stats;
+
+		mutex_init(&wlandev->ioctl_lock);
+		/* block ioctls until fully initialised. Don't forget to call allow_ioctls at some point!*/
+		mutex_lock(&wlandev->ioctl_lock);
+		
 #ifdef HAVE_PRIVATE_IOCTL
 		dev->do_ioctl = 	p80211knetdev_do_ioctl;
 #endif
@@ -929,15 +938,19 @@ int wlan_setup(wlandevice_t *wlandev)
 		dev->open =		p80211knetdev_open;
 		dev->stop =		p80211knetdev_stop;
 
-#ifdef CONFIG_NET_WIRELESS
-#if ((WIRELESS_EXT < 17) && (WIRELESS_EXT < 21))
+#if defined(CONFIG_NET_WIRELESS) || defined(WIRELESS_EXT)
+#if (WIRELESS_EXT < 17)
 		dev->get_wireless_stats = p80211wext_get_wireless_stats;
 #endif
 #if WIRELESS_EXT > 12
 		dev->wireless_handlers = &p80211wext_handler_def;
 #endif
 #endif
-			
+	
+#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24) )
+		dev->nd_net = &init_net;
+#endif
+		
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,38) )
 		dev->tbusy = 1;
 		dev->start = 0;
@@ -1061,9 +1074,23 @@ int register_wlandev(wlandevice_t *wlandev)
 	}
 #endif
 
+	if (wlan_wext_write) {
+/*
+		// fake out a call to ifstate_enable!
+		p80211msg_lnxreq_ifstate_t msg;
+		memset(&msg, 0, sizeof(msg));
+		msg.msgcode = DIDmsg_lnxreq_ifstate;
+		msg.msglen = sizeof(msg);
+		msg.ifstate.status = P80211ENUM_msgitem_status_data_ok;
+		msg.ifstate.data = P80211ENUM_ifstate_enable;
+
+		p80211req_dorequest(wlandev, &msg);
+*/
+	} else {
 #ifdef CONFIG_HOTPLUG
-	p80211_run_sbin_hotplug(wlandev, WLAN_HOTPLUG_REGISTER);
+		p80211_run_sbin_hotplug(wlandev, WLAN_HOTPLUG_REGISTER);
 #endif
+	}
 
 	DBFEXIT;
 	return 0;
@@ -1183,6 +1210,7 @@ p80211netdev_proc_read(
 	p += sprintf(p, "Enabled    : %s%s\n", 
 		     (wlandev->shortpreamble) ? "short_preamble " : "",
 		     (wlandev->hostwep & HOSTWEP_PRIVACYINVOKED) ? "privacy" : "");
+	p += sprintf(p, "msdstate=%d\n", wlandev->msdstate);
 
 
  exit:
@@ -1256,7 +1284,7 @@ void p80211netdev_hwremoved(wlandevice_t *wlandev)
 * Call context:
 *	interrupt
 ----------------------------------------------------------------*/
-int p80211_rx_typedrop( wlandevice_t *wlandev, UINT16 fc)
+static int p80211_rx_typedrop( wlandevice_t *wlandev, UINT16 fc)
 {
 	UINT16	ftype;
 	UINT16	fstype;
@@ -1484,7 +1512,7 @@ void    p80211_resume(wlandevice_t *wlandev)
 	DBFEXIT;
 }
 
-void p80211knetdev_tx_timeout( netdevice_t *netdev) 
+static void p80211knetdev_tx_timeout( netdevice_t *netdev) 
 {
 	wlandevice_t	*wlandev = (wlandevice_t*)netdev->priv;
 	DBFENTER;
@@ -1499,3 +1527,7 @@ void p80211knetdev_tx_timeout( netdevice_t *netdev)
 
 	DBFEXIT;
 }
+
+void p80211_allow_ioctls(wlandevice_t *wlandev) {
+	mutex_unlock(&wlandev->ioctl_lock);
+}
