@@ -37,7 +37,7 @@
 /*================================================================*/
 /* System Includes */
 
-#include <linux/config.h>
+
 #include <linux/version.h>
 
 #include <linux/kernel.h>
@@ -81,7 +81,7 @@ static int p80211wext_giwessid(netdevice_t *dev,
 /* compatibility to wireless extensions */
 #ifdef WIRELESS_EXT
 
-UINT8 p80211_mhz_to_channel(UINT16 mhz)
+static UINT8 p80211_mhz_to_channel(UINT16 mhz)
 {
 	if (mhz >= 5000) {
 		return ((mhz - 5000) / 5);
@@ -97,7 +97,7 @@ UINT8 p80211_mhz_to_channel(UINT16 mhz)
 	return 0;
 }
 
-UINT16 p80211_channel_to_mhz(UINT8 ch, int dot11a)
+static UINT16 p80211_channel_to_mhz(UINT8 ch, int dot11a)
 {
 
 	if (ch == 0)
@@ -124,14 +124,30 @@ UINT16 p80211_channel_to_mhz(UINT8 ch, int dot11a)
 }
 
 /* taken from orinoco.c ;-) */
-const long p80211wext_channel_freq[] = {
+static const long p80211wext_channel_freq[] = {
 	2412, 2417, 2422, 2427, 2432, 2437, 2442,
 	2447, 2452, 2457, 2462, 2467, 2472, 2484
 };
 #define NUM_CHANNELS (sizeof(p80211wext_channel_freq) / sizeof(p80211wext_channel_freq[0])) 
 
+/* steal a spare bit to store the shared/opensystems state. should default to open if not set */
+#define HOSTWEP_SHAREDKEY BIT3
+
+
 /** function declarations =============== */
 
+static int qual_as_percent(int snr ) {
+  if ( snr <= 0 ) 
+    return 0;
+  if ( snr <= 40 )
+    return snr*5/2;
+  return 100;
+}
+
+static inline int invalid_state(wlandevice_t *wlandev ) {
+	return wlandev->msdstate != WLAN_MSD_RUNNING;
+}
+
 static int p80211wext_dorequest(wlandevice_t *wlandev, UINT32 did, UINT32 data)
 {
 	p80211msg_dot11req_mibset_t	msg;
@@ -161,6 +177,12 @@ static int p80211wext_autojoin(wlandevice_t *wlandev)
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
+
 	/* Get ESSID */
 	result = p80211wext_giwessid(wlandev->netdev, NULL, &data, ssid);
 
@@ -169,21 +191,16 @@ static int p80211wext_autojoin(wlandevice_t *wlandev)
 		goto exit;
 	}
 
-#warning "make a smarter sharedkey/opensystem auth decision"
-	/* Get WEPDef */
-	if (wlandev->hostwep & HOSTWEP_PRIVACYINVOKED ||
-	    wlandev->hostwep & HOSTWEP_EXCLUDEUNENCRYPTED) {
-		msg.authtype.data = P80211ENUM_authalg_sharedkey;
-	}
-	else {
-		msg.authtype.data = P80211ENUM_authalg_opensystem;
-	}
+	if ( wlandev->hostwep & HOSTWEP_SHAREDKEY ) 
+	  msg.authtype.data = P80211ENUM_authalg_sharedkey;
+	else
+	  msg.authtype.data = P80211ENUM_authalg_opensystem;
 
 	msg.msgcode = DIDmsg_lnxreq_autojoin;		
 
 	/* Trim the last '\0' to fit the SSID format */
 
-	if (ssid[data.length-1] == '\0') {
+	if (data.length && ssid[data.length-1] == '\0') {
 		data.length = data.length - 1;
 	}
 	
@@ -213,7 +230,6 @@ struct iw_statistics* p80211wext_get_wireless_stats (netdevice_t *dev)
 	int retval;
 	
 	DBFENTER;
-	
 	/* Check */
 	if ( (wlandev == NULL) || (wlandev->msdstate != WLAN_MSD_RUNNING) )
 		return NULL;
@@ -232,11 +248,15 @@ struct iw_statistics* p80211wext_get_wireless_stats (netdevice_t *dev)
 
 	retval = wlandev->mlmerequest(wlandev, (p80211msg_t*) &quality);
 
-	wstats->qual.qual = quality.link.data;    /* overall link quality */
+	wstats->qual.qual = qual_as_percent(quality.link.data);    /* overall link quality */
 	wstats->qual.level = quality.level.data;  /* instant signal level */
 	wstats->qual.noise = quality.noise.data;  /* instant noise level */
 
+#if WIRELESS_EXT > 18
+	wstats->qual.updated = IW_QUAL_ALL_UPDATED | IW_QUAL_DBM;
+#else
 	wstats->qual.updated = 7;
+#endif
 	wstats->discard.code = wlandev->rx.decrypt_err;
 	wstats->discard.nwid = 0;
 	wstats->discard.misc = 0;
@@ -295,6 +315,12 @@ static int p80211wext_giwfreq(netdevice_t *dev,
 	int err = 0;
 
 	DBFENTER;
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
+
 
 	msg.msgcode = DIDmsg_dot11req_mibget;
 	mibitem.did = DIDmib_dot11phy_dot11PhyDSSSTable_dot11CurrentChannel;
@@ -334,6 +360,12 @@ static int p80211wext_siwfreq(netdevice_t *dev,
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
+
 	if (!wlan_wext_write) {
 		err = (-EOPNOTSUPP);
 		goto exit;
@@ -402,6 +434,11 @@ static int p80211wext_siwmode(netdevice_t *dev,
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 	if (!wlan_wext_write) {
 		err = (-EOPNOTSUPP);
 		goto exit;
@@ -462,6 +499,10 @@ static int p80211wext_giwrange(netdevice_t *dev,
 
 	DBFENTER;
 
+	// for backward compatability set size & zero everything we don't understand
+	data->length = sizeof(*range);
+	memset(range,0,sizeof(*range));
+
 #if WIRELESS_EXT > 9
 	range->txpower_capa = IW_TXPOW_DBM;
 	// XXX what about min/max_pmp, min/max_pmt, etc.
@@ -477,6 +518,15 @@ static int p80211wext_giwrange(netdevice_t *dev,
 	range->max_retry = 255;
 #endif /* WIRELESS_EXT > 10 */
 
+#if WIRELESS_EXT > 16
+        range->event_capa[0] = (IW_EVENT_CAPA_K_0 |  //mode/freq/ssid 
+                                IW_EVENT_CAPA_MASK(SIOCGIWAP) | 
+                                IW_EVENT_CAPA_MASK(SIOCGIWSCAN));
+        range->event_capa[1] = IW_EVENT_CAPA_K_1;  //encode
+        range->event_capa[4] = (IW_EVENT_CAPA_MASK(IWEVQUAL) |
+                                IW_EVENT_CAPA_MASK(IWEVCUSTOM) );
+#endif
+
 	range->num_channels = NUM_CHANNELS;
 
 	/* XXX need to filter against the regulatory domain &| active set */
@@ -491,9 +541,9 @@ static int p80211wext_giwrange(netdevice_t *dev,
 	range->num_frequency = val;
 	
 	/* Max of /proc/net/wireless */
-	range->max_qual.qual = 92;
-	range->max_qual.level = 154;
-	range->max_qual.noise = 154;
+	range->max_qual.qual = 100;
+	range->max_qual.level = 0;
+	range->max_qual.noise = 0;
 	range->sensitivity = 3;
 	// XXX these need to be nsd-specific!
 
@@ -546,18 +596,19 @@ static int p80211wext_giwencode(netdevice_t *dev,
 
 	DBFENTER;
 
+	i = (erq->flags & IW_ENCODE_INDEX) - 1;
+	erq->flags = 0;
+
 	if (wlandev->hostwep & HOSTWEP_PRIVACYINVOKED)
-		erq->flags = IW_ENCODE_ENABLED;
+		erq->flags |= IW_ENCODE_ENABLED;
 	else
-		erq->flags = IW_ENCODE_DISABLED;	
+		erq->flags |= IW_ENCODE_DISABLED;	
 
 	if (wlandev->hostwep & HOSTWEP_EXCLUDEUNENCRYPTED)
 		erq->flags |= IW_ENCODE_RESTRICTED;
 	else
 		erq->flags |= IW_ENCODE_OPEN;
 
-	i = (erq->flags & IW_ENCODE_INDEX) - 1;
-
 	if (i == -1)
 		i = wlandev->hostwep & HOSTWEP_DEFAULTKEY_MASK;
 
@@ -587,10 +638,14 @@ static int p80211wext_siwencode(netdevice_t *dev,
 
 	int err = 0;
 	int result = 0;
-	int enable = 0;
 	int i;
 
 	DBFENTER;
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 
 	if (!wlan_wext_write) {
 		err = (-EOPNOTSUPP);
@@ -603,27 +658,27 @@ static int p80211wext_siwencode(netdevice_t *dev,
 		if ((i < 1) || (i > NUM_WEPKEYS)) {
 			err = -EINVAL;
 			goto exit;	
-		}		
-		else
+		} else {
 			i--;
+		}
 
-		result = p80211wext_dorequest(wlandev, DIDmib_dot11smt_dot11PrivacyTable_dot11WEPDefaultKeyID, i);
+		/* Set current key number only if no keys are given */
+		if (erq->flags & IW_ENCODE_NOKEY) {
+			result = p80211wext_dorequest(wlandev, DIDmib_dot11smt_dot11PrivacyTable_dot11WEPDefaultKeyID, i);
 		
-		if (result) {
-			err = -EFAULT;
-			goto exit;
-		}
-		else {
-			enable = 1;
+			if (result) {
+				err = -EFAULT;
+				goto exit;
+			}
 		}
 
-	}
-	else {
-		// Do not thing when no Key Index
+	} else {
+		// Use defaultkey if no Key Index
+		i = wlandev->hostwep & HOSTWEP_DEFAULTKEY_MASK;
 	}
 
 	/* Check if there is no key information in the iwconfig request */
-	if((erq->flags & IW_ENCODE_NOKEY) == 0 && enable == 1) {
+	if((erq->flags & IW_ENCODE_NOKEY) == 0 ) {
 
 		/*------------------------------------------------------------ 
 		 * If there is WEP Key for setting, check the Key Information 
@@ -678,8 +733,7 @@ static int p80211wext_siwencode(netdevice_t *dev,
 	/* Check the PrivacyInvoked flag */
 	if (erq->flags & IW_ENCODE_DISABLED) {
 		result = p80211wext_dorequest(wlandev, DIDmib_dot11smt_dot11PrivacyTable_dot11PrivacyInvoked, P80211ENUM_truth_false);
-	}
-	else if((erq->flags & IW_ENCODE_ENABLED) || enable == 1) {
+	} else {
 		result = p80211wext_dorequest(wlandev, DIDmib_dot11smt_dot11PrivacyTable_dot11PrivacyInvoked, P80211ENUM_truth_true);
 	}
 
@@ -688,12 +742,19 @@ static int p80211wext_siwencode(netdevice_t *dev,
 		goto exit;
 	}
 
-	/* Check the ExcludeUnencrypted flag */
+	/*  The  security  mode  may  be open or restricted, and its meaning
+	    depends on the card used. With  most  cards,  in  open  mode  no
+	    authentication  is  used  and  the  card  may  also  accept non-
+	    encrypted sessions, whereas in restricted  mode  only  encrypted
+	    sessions  are  accepted  and the card will use authentication if
+	    available. 
+	*/
 	if (erq->flags & IW_ENCODE_RESTRICTED) {
 		result = p80211wext_dorequest(wlandev, DIDmib_dot11smt_dot11PrivacyTable_dot11ExcludeUnencrypted, P80211ENUM_truth_true);
-	}
-	else if (erq->flags & IW_ENCODE_OPEN) {
+		// wlandev->hostwep |= HOSTWEP_SHAREDKEY;
+	} else if (erq->flags & IW_ENCODE_OPEN) {
 		result = p80211wext_dorequest(wlandev, DIDmib_dot11smt_dot11PrivacyTable_dot11ExcludeUnencrypted, P80211ENUM_truth_false);
+		// wlandev->hostwep &= ~HOSTWEP_SHAREDKEY;
 	}
 
 	if (result) {
@@ -715,9 +776,19 @@ static int p80211wext_giwessid(netdevice_t *dev,
 
 	DBFENTER;
 
-	data->length = wlandev->ssid.len;
-	data->flags = 1;
-	memcpy(essid, wlandev->ssid.data, data->length);
+	if (wlandev->ssid.len) {
+		data->length = wlandev->ssid.len;
+		data->flags = 1;
+		memcpy(essid, wlandev->ssid.data, data->length);
+		essid[data->length] = 0;
+#if (WIRELESS_EXT < 21)
+		data->length++;
+#endif
+	} else {
+	  	memset(essid, 0, sizeof(wlandev->ssid.data));
+		data->length = 0;
+		data->flags = 0;
+	}
 
 	DBFEXIT;
 	return 0;
@@ -728,44 +799,54 @@ static int p80211wext_siwessid(netdevice_t *dev,
 			       struct iw_point *data, char *essid)
 {
 	wlandevice_t *wlandev = (wlandevice_t*)dev->priv;
-	p80211msg_dot11req_mibset_t     msg;
-	p80211item_pstr32_t             pstr;
+	p80211msg_lnxreq_autojoin_t     msg;
 
 	int result;
 	int err = 0;
+	int length = data->length;
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 	if (!wlan_wext_write) {
 		err = (-EOPNOTSUPP);
 		goto exit;
 	}
 
-	memset(&msg, 0, sizeof(msg));
-	memset(&pstr, 0, sizeof(pstr));  
-	msg.msgcode = DIDmsg_dot11req_mibset;
 
-#warning "Get rid of p2mib here!"
-
-	if (wlandev->macmode == WLAN_MACMODE_ESS_AP)
-		pstr.did = DIDmib_p2_p2Static_p2CnfOwnSSID;
+	if ( wlandev->hostwep & HOSTWEP_SHAREDKEY ) 
+	  msg.authtype.data = P80211ENUM_authalg_sharedkey;
 	else
-		pstr.did = DIDmib_dot11smt_dot11StationConfigTable_dot11DesiredSSID;
+	  msg.authtype.data = P80211ENUM_authalg_opensystem;
 
-	pstr.data.len = data->length;
-	memcpy(pstr.data.data, essid, data->length);
+	msg.msgcode = DIDmsg_lnxreq_autojoin;		
 
-	memcpy(&msg.mibattribute.data, &pstr, sizeof(pstr));
+#if (WIRELESS_EXT < 21)
+	if (length) length--;
+#endif
+	
+	/* Trim the last '\0' to fit the SSID format */
+
+	if (length && essid[length-1] == '\0') {
+	  length--;
+	}
+	
+	memcpy(msg.ssid.data.data, essid, length);
+	msg.ssid.data.len = length;
+
+	WLAN_LOG_DEBUG(1,"autojoin_ssid for %s \n",essid);
 	result = p80211req_dorequest(wlandev, (UINT8*)&msg);
+        WLAN_LOG_DEBUG(1,"autojoin_ssid %d\n",result);
 
 	if (result) {
 		err = -EFAULT;
 		goto exit;
 	}
 
-	/* Auto Join */
-	err = p80211wext_autojoin(wlandev); 
-
  exit:
 	DBFEXIT;
 	return err;
@@ -781,6 +862,11 @@ static int p80211wext_siwcommit(netdevice_t *dev,
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 	if (!wlan_wext_write) {
 		err = (-EOPNOTSUPP);
 		goto exit;
@@ -807,6 +893,11 @@ static int p80211wext_giwrate(netdevice_t *dev,
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 	msg.msgcode = DIDmsg_dot11req_mibget;
 	mibitem.did = DIDmib_p2_p2MAC_p2CurrentTxRate;
 	memcpy(&msg.mibattribute.data, &mibitem, sizeof(mibitem));
@@ -861,6 +952,11 @@ static int p80211wext_giwrts(netdevice_t *dev,
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 	msg.msgcode = DIDmsg_dot11req_mibget;
 	mibitem.did = DIDmib_dot11mac_dot11OperationTable_dot11RTSThreshold;
 	memcpy(&msg.mibattribute.data, &mibitem, sizeof(mibitem));
@@ -895,6 +991,11 @@ static int p80211wext_siwrts(netdevice_t *dev,
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 	if (!wlan_wext_write) {
 		err = (-EOPNOTSUPP);
 		goto exit;
@@ -932,6 +1033,11 @@ static int p80211wext_giwfrag(netdevice_t *dev,
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 	msg.msgcode = DIDmsg_dot11req_mibget;
 	mibitem.did = DIDmib_dot11mac_dot11OperationTable_dot11FragmentationThreshold;
 	memcpy(&msg.mibattribute.data, &mibitem, sizeof(mibitem));
@@ -965,6 +1071,11 @@ static int p80211wext_siwfrag(netdevice_t *dev,
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 	if (!wlan_wext_write) {
 		err = (-EOPNOTSUPP);
 		goto exit;
@@ -994,6 +1105,15 @@ static int p80211wext_siwfrag(netdevice_t *dev,
 #endif  /* WIRELESS_EXT > 8 */
 
 #if WIRELESS_EXT > 10
+
+#ifndef IW_RETRY_LONG
+#define IW_RETRY_LONG IW_RETRY_MAX
+#endif
+
+#ifndef IW_RETRY_SHORT
+#define IW_RETRY_SHORT IW_RETRY_MIN
+#endif
+
 static int p80211wext_giwretry(netdevice_t *dev,
 			       struct iw_request_info *info,
 			       struct iw_param *rrq, char *extra)
@@ -1006,6 +1126,11 @@ static int p80211wext_giwretry(netdevice_t *dev,
 	UINT16 shortretry, longretry, lifetime;
 
 	DBFENTER;
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 
 	msg.msgcode = DIDmsg_dot11req_mibget;
 	mibitem.did = DIDmib_dot11mac_dot11OperationTable_dot11ShortRetryLimit;
@@ -1056,14 +1181,14 @@ static int p80211wext_giwretry(netdevice_t *dev,
 		rrq->flags = IW_RETRY_LIFETIME;
 		rrq->value = lifetime * 1024;
 	} else {
-		if (rrq->flags & IW_RETRY_MAX) {
-			rrq->flags = IW_RETRY_LIMIT | IW_RETRY_MAX;
+		if (rrq->flags & IW_RETRY_LONG) {
+			rrq->flags = IW_RETRY_LIMIT | IW_RETRY_LONG;
 			rrq->value = longretry;
 		} else {
 			rrq->flags = IW_RETRY_LIMIT;
 			rrq->value = shortretry;
 			if (shortretry != longretry)
-				rrq->flags |= IW_RETRY_MIN;
+				rrq->flags |= IW_RETRY_SHORT;
 		}
 	}
 
@@ -1085,6 +1210,11 @@ static int p80211wext_siwretry(netdevice_t *dev,
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 	if (!wlan_wext_write) {
 		err = (-EOPNOTSUPP);
 		goto exit;
@@ -1109,7 +1239,7 @@ static int p80211wext_siwretry(netdevice_t *dev,
 			goto exit;
 		}
 	} else {
-		if (rrq->flags & IW_RETRY_MAX) {
+		if (rrq->flags & IW_RETRY_LONG) {
 			mibitem.did = DIDmib_dot11mac_dot11OperationTable_dot11LongRetryLimit;
 			mibitem.data = rrq->value;
 
@@ -1122,7 +1252,7 @@ static int p80211wext_siwretry(netdevice_t *dev,
 			}
 		}
 
-		if (rrq->flags & IW_RETRY_MIN) {
+		if (rrq->flags & IW_RETRY_SHORT) {
 			mibitem.did = DIDmib_dot11mac_dot11OperationTable_dot11ShortRetryLimit;
 			mibitem.data = rrq->value;
 
@@ -1157,6 +1287,11 @@ static int p80211wext_siwtxpow(netdevice_t *dev,
                               
         DBFENTER;
 	
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
        if (!wlan_wext_write) {
                 err = (-EOPNOTSUPP); 
                 goto exit;    
@@ -1202,6 +1337,11 @@ static int p80211wext_giwtxpow(netdevice_t *dev,
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 	msg.msgcode = DIDmsg_dot11req_mibget;
 	mibitem.did = DIDmib_dot11phy_dot11PhyTxPowerTable_dot11CurrentTxPowerLevel;
 
@@ -1302,7 +1442,7 @@ static int p80211wext_giwspy(netdevice_t *dev,
 	return 0;
 }
 
-int prism2_result2err (int prism2_result)
+static int prism2_result2err (int prism2_result)
 {
 	int err = 0;
 
@@ -1337,6 +1477,11 @@ static int p80211wext_siwscan(netdevice_t *dev,
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
 	if (wlandev->macmode == WLAN_MACMODE_ESS_AP) {
 		WLAN_LOG_ERROR("Can't scan in AP mode\n");
 		err = (-EOPNOTSUPP);
@@ -1353,8 +1498,8 @@ static int p80211wext_siwscan(netdevice_t *dev,
 	msg.scantype.data = P80211ENUM_scantype_active;
 	msg.probedelay.data = 0;
 
-	for (i = 0; i < 14; i++)
-		msg.channellist.data.data[i] = i;
+	for (i = 1; i <= 14; i++)
+		msg.channellist.data.data[i-1] = i;
 	msg.channellist.data.len = 14;
 
 	msg.maxchanneltime.data = 250;
@@ -1439,7 +1584,7 @@ wext_translate_bss(char *current_ev, char *end_buf, p80211msg_dot11req_scan_resu
 	iwe.u.qual.level = bss->signal.data;
 	iwe.u.qual.noise = bss->noise.data;
 	/* do a simple SNR for quality */
-	iwe.u.qual.qual = bss->signal.data - bss->noise.data;
+	iwe.u.qual.qual = qual_as_percent(bss->signal.data - bss->noise.data);
 	iwe.cmd = IWEVQUAL;
 	current_ev = iwe_stream_add_event(current_ev, end_buf, &iwe, IW_EV_QUAL_LEN);
 
@@ -1461,6 +1606,12 @@ static int p80211wext_giwscan(netdevice_t *dev,
 
 	DBFENTER;
 
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		err = -EBUSY;
+		goto exit;
+	}
+
 	/* Since wireless tools doesn't really have a way of passing how
 	 * many scan results results there were back here, keep grabbing them
 	 * until we fail.
@@ -1486,12 +1637,270 @@ static int p80211wext_giwscan(netdevice_t *dev,
 
 	if (result && !scan_good)
 		err = prism2_result2err (msg.resultcode.data);
-
+  exit:
 	DBFEXIT;
 	return err;
 }
 #endif
 
+/*****************************************************/
+//extra wireless extensions stuff to support NetworkManager (I hope)
+
+#if WIRELESS_EXT > 17
+/* SIOCSIWENCODEEXT */
+static int p80211wext_set_encodeext(struct net_device *dev,
+				struct iw_request_info *info,
+				union iwreq_data *wrqu, char *extra)
+{
+  wlandevice_t *wlandev = (wlandevice_t*)dev->priv;
+  struct iw_encode_ext *ext = (struct iw_encode_ext *)extra;
+	p80211msg_dot11req_mibset_t	msg;
+	p80211item_pstr32_t		*pstr;
+
+  int result = 0;
+  struct iw_point *encoding = &wrqu->encoding;
+  int idx = encoding->flags & IW_ENCODE_INDEX;
+
+  DBFENTER;
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		result = -EBUSY;
+		goto exit;
+	}
+
+  WLAN_LOG_DEBUG(1,"set_encode_ext flags[%d] alg[%d] keylen[%d]\n",ext->ext_flags,(int)ext->alg,(int)ext->key_len);
+
+
+  if ( ext->ext_flags & IW_ENCODE_EXT_GROUP_KEY ) {
+    // set default key ? I'm not sure if this the the correct thing to do here 
+
+    if ( idx ) {
+      if (idx < 1 || idx > NUM_WEPKEYS) {
+	return -EINVAL;
+      } else 
+	idx--;
+    }
+    WLAN_LOG_DEBUG(1,"setting default key (%d)\n",idx);
+    result = p80211wext_dorequest(wlandev, DIDmib_dot11smt_dot11PrivacyTable_dot11WEPDefaultKeyID, idx);
+    if ( result ) 
+      return -EFAULT;
+  }
+
+
+  if ( ext->ext_flags & IW_ENCODE_EXT_SET_TX_KEY ) {
+    if ( ! ext->alg & IW_ENCODE_ALG_WEP) {
+      WLAN_LOG_DEBUG(1,"asked to set a non wep key :(");
+      return -EINVAL;
+    }
+    if (idx) {
+      if (idx <1 || idx > NUM_WEPKEYS) 
+	return -EINVAL;
+      else
+	idx--;
+    }
+    WLAN_LOG_DEBUG(1,"Set WEP key (%d)\n",idx);
+    wlandev->wep_keylens[idx] = ext->key_len;
+    memcpy(wlandev->wep_keys[idx], ext->key, ext->key_len);
+
+    memset( &msg,0,sizeof(msg));
+    pstr = (p80211item_pstr32_t*)&msg.mibattribute.data;
+    memcpy(pstr->data.data, ext->key,ext->key_len);
+    pstr->data.len = ext->key_len;
+    switch (idx) {
+    case 0:
+      pstr->did = DIDmib_dot11smt_dot11WEPDefaultKeysTable_dot11WEPDefaultKey0;
+      break;
+    case 1:
+      pstr->did = DIDmib_dot11smt_dot11WEPDefaultKeysTable_dot11WEPDefaultKey1;
+      break;
+    case 2:
+      pstr->did = DIDmib_dot11smt_dot11WEPDefaultKeysTable_dot11WEPDefaultKey2;
+      break;
+    case 3:
+      pstr->did = DIDmib_dot11smt_dot11WEPDefaultKeysTable_dot11WEPDefaultKey3;
+      break;
+    default:
+      break;
+    }
+    msg.msgcode = DIDmsg_dot11req_mibset;
+    result = p80211req_dorequest(wlandev,(UINT8*)&msg);
+    WLAN_LOG_DEBUG(1,"result (%d)\n",result);
+  }
+	exit:
+	DBFEXIT;
+  return result;
+}
+
+/* SIOCGIWENCODEEXT */
+static int p80211wext_get_encodeext(struct net_device *dev,
+				struct iw_request_info *info,
+				union iwreq_data *wrqu, char *extra)
+
+{
+	wlandevice_t *wlandev = (wlandevice_t*)dev->priv;
+	struct iw_encode_ext *ext = (struct iw_encode_ext *)extra;
+
+	struct iw_point *encoding = &wrqu->encoding; 
+	int result = 0;
+	int max_len;
+	int idx;
+
+	DBFENTER;
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state  [%d]\n",wlandev->msdstate);
+		result = -EBUSY;
+		goto exit;
+	}
+
+
+	WLAN_LOG_DEBUG(1,"get_encode_ext flags[%d] alg[%d] keylen[%d]\n",ext->ext_flags,(int)ext->alg,(int)ext->key_len);
+
+
+	max_len = encoding->length - sizeof(*ext);
+	if ( max_len <= 0) {
+		WLAN_LOG_DEBUG(1,"get_encodeext max_len [%d] invalid\n",max_len);
+		result = -EINVAL;
+		goto exit;
+	}
+	idx = encoding->flags & IW_ENCODE_INDEX;
+
+	WLAN_LOG_DEBUG(1,"get_encode_ext index [%d]\n",idx);
+
+	if (idx) {
+		if (idx < 1 || idx > NUM_WEPKEYS ) {
+			WLAN_LOG_DEBUG(1,"get_encode_ext invalid key index [%d]\n",idx);
+			result = -EINVAL;
+			goto exit;
+		}
+		idx--;
+	} else {
+		/* default key ? not sure what to do */
+		/* will just use key[0] for now ! FIX ME */
+	}
+
+	encoding->flags = idx + 1;
+	memset(ext,0,sizeof(*ext));
+
+	ext->alg = IW_ENCODE_ALG_WEP;
+	ext->key_len = wlandev->wep_keylens[idx];
+	memcpy( ext->key, wlandev->wep_keys[idx] , ext->key_len );  
+	
+	encoding->flags |= IW_ENCODE_ENABLED;
+exit:
+	DBFEXIT;
+
+	return result;
+}
+
+
+/* SIOCSIWAUTH */
+static int p80211_wext_set_iwauth (struct net_device *dev,
+				   struct iw_request_info *info,
+				   union iwreq_data *wrqu, char *extra)
+{
+  wlandevice_t *wlandev = (wlandevice_t*)dev->priv;
+  struct iw_param *param = &wrqu->param;
+  int result =0;
+
+	DBFENTER;
+	if ( invalid_state(wlandev) ) {
+		WLAN_LOG_DEBUG(1,"called from invalid state [%d]\n",wlandev->msdstate);
+		result = -EBUSY;
+		goto exit;
+	}
+
+
+  WLAN_LOG_DEBUG(1,"set_iwauth flags[%d]\n",(int)param->flags & IW_AUTH_INDEX );
+
+  switch (param->flags & IW_AUTH_INDEX) {
+  case IW_AUTH_DROP_UNENCRYPTED:
+    WLAN_LOG_DEBUG(1,"drop_unencrypted %d\n",param->value);
+    if (param->value) 
+      result = p80211wext_dorequest(wlandev, DIDmib_dot11smt_dot11PrivacyTable_dot11ExcludeUnencrypted, P80211ENUM_truth_true);
+    else 
+      result = p80211wext_dorequest(wlandev, DIDmib_dot11smt_dot11PrivacyTable_dot11ExcludeUnencrypted, P80211ENUM_truth_false);
+    break;
+
+  case IW_AUTH_PRIVACY_INVOKED:
+    WLAN_LOG_DEBUG(1,"privacy invoked %d\n",param->value);
+    if ( param->value)
+      result = p80211wext_dorequest(wlandev, DIDmib_dot11smt_dot11PrivacyTable_dot11PrivacyInvoked, P80211ENUM_truth_true);
+    else
+      result = p80211wext_dorequest(wlandev, DIDmib_dot11smt_dot11PrivacyTable_dot11PrivacyInvoked, P80211ENUM_truth_false);
+  
+    break;
+
+  case IW_AUTH_80211_AUTH_ALG:
+    if ( param->value & IW_AUTH_ALG_OPEN_SYSTEM ) {
+      WLAN_LOG_DEBUG(1,"set open_system\n");
+      wlandev->hostwep &= ~HOSTWEP_SHAREDKEY;
+    } else if ( param->value & IW_AUTH_ALG_SHARED_KEY) {
+      WLAN_LOG_DEBUG(1,"set shared key\n");
+      wlandev->hostwep |= HOSTWEP_SHAREDKEY;
+    } else {
+      /* don't know what to do know :( */
+      WLAN_LOG_DEBUG(1,"unknown AUTH_ALG (%d)\n",param->value);
+      result = -EINVAL;
+    }
+    break;
+
+  default:
+    break;
+  }
+
+	exit:
+	DBFEXIT;
+  return result;
+}
+
+/* SIOCSIWAUTH */
+static int p80211_wext_get_iwauth (struct net_device *dev,
+				   struct iw_request_info *info,
+				   union iwreq_data *wrqu, char *extra)
+{
+  wlandevice_t *wlandev = (wlandevice_t*)dev->priv;
+  struct iw_param *param = &wrqu->param;
+  int result =0;
+
+	DBFENTER;
+  WLAN_LOG_DEBUG(1,"get_iwauth flags[%d]\n",(int)param->flags & IW_AUTH_INDEX );
+
+  switch (param->flags & IW_AUTH_INDEX) {
+  case IW_AUTH_DROP_UNENCRYPTED:
+    param->value = wlandev->hostwep & HOSTWEP_EXCLUDEUNENCRYPTED?1:0;
+    break;
+
+  case IW_AUTH_PRIVACY_INVOKED:
+    param->value = wlandev->hostwep & HOSTWEP_PRIVACYINVOKED?1:0;
+    break;
+
+  case IW_AUTH_80211_AUTH_ALG:
+    param->value = wlandev->hostwep & HOSTWEP_SHAREDKEY?IW_AUTH_ALG_SHARED_KEY:IW_AUTH_ALG_OPEN_SYSTEM;
+    break;
+
+
+  default:
+    break;
+  }
+
+	DBFEXIT;
+	
+  return result;
+}
+
+
+#endif
+
+
+
+
+
+
+/*****************************************************/
+
+
+
+
 
 /*
 typedef int (*iw_handler)(netdevice_t *dev, struct iw_request_info *info,
@@ -1551,6 +1960,20 @@ static iw_handler p80211wext_handlers[] =  {
 	(iw_handler) p80211wext_giwencode,  		/* SIOCGIWENCODE */
 	(iw_handler) NULL,                 		/* SIOCSIWPOWER */
 	(iw_handler) NULL,                  		/* SIOCGIWPOWER */
+#if WIRELESS_EXT > 17
+/* WPA operations */
+
+	(iw_handler) NULL,				/* -- hole -- */
+	(iw_handler) NULL,				/* -- hole -- */
+	(iw_handler) NULL, /* SIOCSIWGENIE	set generic IE */
+	(iw_handler) NULL, /* SIOCGIWGENIE	get generic IE */
+	(iw_handler) p80211_wext_set_iwauth, /* SIOCSIWAUTH	set authentication mode params */
+	(iw_handler) p80211_wext_get_iwauth, /* SIOCGIWAUTH	get authentication mode params */
+
+	(iw_handler) p80211wext_set_encodeext, /* SIOCSIWENCODEEXT  set encoding token & mode */
+	(iw_handler) p80211wext_get_encodeext, /* SIOCGIWENCODEEXT  get encoding token & mode */
+	(iw_handler) NULL, /* SIOCSIWPMKSA	PMKSA cache operation */
+#endif
 };
 
 struct iw_handler_def p80211wext_handler_def = {
@@ -1715,6 +2138,34 @@ int p80211wext_support_ioctl(netdevice_t *dev, struct ifreq *ifr, int cmd)
 	return (err);
 }
 
+int p80211wext_event_associated(wlandevice_t *wlandev, int assoc)
+{
+        union iwreq_data data;
+
+        DBFENTER;
+
+#if WIRELESS_EXT > 13
+        /* Send the association state first */
+        data.ap_addr.sa_family = ARPHRD_ETHER;
+        if (assoc) {
+                memcpy(data.ap_addr.sa_data, wlandev->bssid, WLAN_ADDR_LEN);
+        } else {
+                memset(data.ap_addr.sa_data, 0, WLAN_ADDR_LEN);
+        }       
+
+        if (wlan_wext_write)
+                wireless_send_event(wlandev->netdev, SIOCGIWAP, &data, NULL);
+
+        if (!assoc) goto done;
+
+        // XXX send association data, like IEs, etc etc.
+#endif
+ done:
+        DBFEXIT;
+        return 0;
+}
+
+
 #endif /* compatibility to wireless extensions */
 
 
