@@ -41,7 +41,7 @@ static int prism2_cs_probe(struct pcmcia_device *pdev);
 #else
 dev_link_t	*prism2sta_attach(void);
 static void	prism2sta_detach(dev_link_t *link);
-static int	prism2sta_config(dev_link_t *link);
+static void	prism2sta_config(dev_link_t *link);
 static void	prism2sta_release(u_long arg);
 static int 	prism2sta_event (event_t event, int priority, event_callback_args_t *args);
 
@@ -173,7 +173,7 @@ if (ret != 0) { \
 } \
 } while (0)
 
-void prism2_cs_remove(struct pcmcia_device *pdev)
+static void prism2_cs_remove(struct pcmcia_device *pdev)
 {
 	struct wlandevice  *wlandev;
 
@@ -228,7 +228,7 @@ void prism2_cs_remove(struct pcmcia_device *pdev)
 	return;
 }
 
-int prism2_cs_suspend(struct pcmcia_device *pdev)
+static int prism2_cs_suspend(struct pcmcia_device *pdev)
 {
 	struct wlandevice  *wlandev;
 
@@ -243,21 +243,20 @@ int prism2_cs_suspend(struct pcmcia_device *pdev)
 	prism2sta_ifstate(wlandev, P80211ENUM_ifstate_disable);
 #else
 	wlandev = link->priv;
-#endif
-	wlandev = link->priv;
 
         link->state |= DEV_SUSPEND;
         if (link->state & DEV_CONFIG) {	
 		prism2sta_ifstate(wlandev, P80211ENUM_ifstate_disable);
 		pcmcia_release_configuration(link->handle);
 	}
+#endif
 
 	DBFEXIT;
 
 	return 0;
 }
 
-int prism2_cs_resume(struct pcmcia_device *pdev)
+static int prism2_cs_resume(struct pcmcia_device *pdev)
 {
 	struct wlandevice  *wlandev;
 
@@ -285,7 +284,7 @@ int prism2_cs_resume(struct pcmcia_device *pdev)
 	return 0;
 }
 
-int prism2_cs_probe(struct pcmcia_device *pdev)
+static int prism2_cs_probe(struct pcmcia_device *pdev)
 {
 	int rval = 0;
 	struct wlandevice *wlandev = NULL;
@@ -520,12 +519,6 @@ int prism2_cs_probe(struct pcmcia_device *pdev)
 	/* Register the network device and get assigned a name */
 	SET_MODULE_OWNER(wlandev->netdev);
 	SET_NETDEV_DEV(wlandev->netdev,  &handle_to_dev(pdev));
-	if (register_wlandev(wlandev) != 0) {
-		WLAN_LOG_NOTICE("prism2sta_cs: register_wlandev() failed.\n");
-		goto failed;
-	}
-
-	strcpy(hw->node.dev_name, wlandev->name);
 
 	/* Allocate an interrupt line.  Note that this does not assign a */
 	/* handler to the interrupt, unless the 'Handler' member of the */
@@ -578,6 +571,15 @@ int prism2_cs_probe(struct pcmcia_device *pdev)
 	/* And now we're done! */
 	wlandev->msdstate = WLAN_MSD_HWPRESENT;
 
+	if (register_wlandev(wlandev) != 0) {
+		WLAN_LOG_NOTICE("prism2sta_cs: register_wlandev() failed.\n");
+		goto failed;
+	}
+	strcpy(hw->node.dev_name, wlandev->name);
+
+	if (wlan_wext_write)
+		prism2sta_ifstate(wlandev, P80211ENUM_ifstate_enable);
+
 	goto done;
 
  cs_failed:
@@ -597,7 +599,7 @@ failed:
 	if (wlandev) {
 		wlan_unsetup(wlandev);
 		if (wlandev->priv) {
-			hfa384x_t *hw = wlandev->priv;
+			hw = wlandev->priv;
 			wlandev->priv = NULL;
 			if (hw) {
 				hfa384x_destroy(hw);
@@ -609,6 +611,7 @@ failed:
 
 done:
 	if (parse) kfree(parse);
+	p80211_allow_ioctls(wlandev);
 	       
 	DBFEXIT;
 	return rval;
@@ -1080,11 +1083,25 @@ next_entry:
 	/* Let pcmcia know the device name */
 	link->dev = &hw->node;
 
-	/* Register the network device and get assigned a name */
+	if (prism2_doreset) {
+		result = hfa384x_corereset(hw, 
+					   prism2_reset_holdtime, 
+					   prism2_reset_settletime, 0);
+		if ( result ) {
+			WLAN_LOG_ERROR(
+				"corereset() failed, result=%d.\n", 
+				result);
+			wlandev->msdstate = WLAN_MSD_HWFAIL;
+			goto failed;
+		}
+	}
+
 	SET_MODULE_OWNER(wlandev->netdev);
 #if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11) )
 	SET_NETDEV_DEV(wlandev->netdev,  &handle_to_dev(link->handle));
 #endif
+
+	/* Register the network device and get assigned a name */
 	if (register_wlandev(wlandev) != 0) {
 		WLAN_LOG_NOTICE("prism2sta_cs: register_wlandev() failed.\n");
 		goto failed;
@@ -1217,19 +1234,6 @@ prism2sta_event (
 		hw->iobase = wlandev->netdev->base_addr;
 		hw->link = link;
 
-		if (prism2_doreset) {
-			result = hfa384x_corereset(hw, 
-					prism2_reset_holdtime, 
-					prism2_reset_settletime, 0);
-			if ( result ) {
-				WLAN_LOG_ERROR(
-					"corereset() failed, result=%d.\n", 
-					result);
-				wlandev->msdstate = WLAN_MSD_HWFAIL;
-				break;
-			}
-		}
-
 #if 0
 		/*
 		 * TODO: test_hostif() not implemented yet.
