@@ -56,7 +56,7 @@
 
 #include <wlan/version.h>
 
-#include <linux/config.h>
+
 #include <linux/version.h>
 
 #include <linux/module.h>
@@ -295,7 +295,7 @@ inline void dmpmem(void *buf, int n)
 * Call context:
 *	process thread
 ----------------------------------------------------------------*/
-int prism2sta_open(wlandevice_t *wlandev)
+static int prism2sta_open(wlandevice_t *wlandev)
 {
 	DBFENTER;
 
@@ -337,7 +337,7 @@ int prism2sta_open(wlandevice_t *wlandev)
 * Call context:
 *	process thread
 ----------------------------------------------------------------*/
-int prism2sta_close(wlandevice_t *wlandev)
+static int prism2sta_close(wlandevice_t *wlandev)
 {
 	DBFENTER;
 
@@ -373,7 +373,7 @@ int prism2sta_close(wlandevice_t *wlandev)
 * Call context:
 *	process thread
 ----------------------------------------------------------------*/
-void prism2sta_reset(wlandevice_t *wlandev )
+static void prism2sta_reset(wlandevice_t *wlandev )
 {
 	DBFENTER;
 	DBFEXIT;
@@ -402,8 +402,9 @@ void prism2sta_reset(wlandevice_t *wlandev )
 * Call context:
 *	process thread
 ----------------------------------------------------------------*/
-int prism2sta_txframe(wlandevice_t *wlandev, struct sk_buff *skb,
-		      p80211_hdr_t *p80211_hdr, p80211_metawep_t *p80211_wep)
+static int prism2sta_txframe(wlandevice_t *wlandev, struct sk_buff *skb,
+			     p80211_hdr_t *p80211_hdr,
+			     p80211_metawep_t *p80211_wep)
 {
 	hfa384x_t		*hw = (hfa384x_t *)wlandev->priv;
 	int			result;
@@ -445,7 +446,7 @@ int prism2sta_txframe(wlandevice_t *wlandev, struct sk_buff *skb,
 * Call context:
 *	process thread
 ----------------------------------------------------------------*/
-int prism2sta_mlmerequest(wlandevice_t *wlandev, p80211msg_t *msg)
+static int prism2sta_mlmerequest(wlandevice_t *wlandev, p80211msg_t *msg)
 {
         hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
 
@@ -836,7 +837,7 @@ UINT32 prism2sta_ifstate(wlandevice_t *wlandev, UINT32 ifstate)
 * Call context:
 *	Either.
 ----------------------------------------------------------------*/
-int prism2sta_getcardinfo(wlandevice_t *wlandev)
+static int prism2sta_getcardinfo(wlandevice_t *wlandev)
 {
 	int 			result = 0;
         hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
@@ -1153,7 +1154,7 @@ done:
 * Call context:
 *	process thread
 ----------------------------------------------------------------*/
-int prism2sta_globalsetup(wlandevice_t *wlandev)
+static int prism2sta_globalsetup(wlandevice_t *wlandev)
 {
 	hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
 
@@ -1162,7 +1163,7 @@ int prism2sta_globalsetup(wlandevice_t *wlandev)
 	                                    WLAN_DATA_MAXLEN);
 }
 
-int prism2sta_setmulticast(wlandevice_t *wlandev, netdevice_t *dev)
+static int prism2sta_setmulticast(wlandevice_t *wlandev, netdevice_t *dev)
 {
 	int result = 0;
 	hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
@@ -1217,7 +1218,7 @@ int prism2sta_setmulticast(wlandevice_t *wlandev, netdevice_t *dev)
 * Call context:
 *	interrupt
 ----------------------------------------------------------------*/
-void prism2sta_inf_handover(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
+static void prism2sta_inf_handover(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
 {
 	DBFENTER;
 	WLAN_LOG_DEBUG(2,"received infoframe:HANDOVER (unhandled)\n");
@@ -1243,7 +1244,7 @@ void prism2sta_inf_handover(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
 * Call context:
 *	interrupt
 ----------------------------------------------------------------*/
-void prism2sta_inf_tallies(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
+static void prism2sta_inf_tallies(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
 {
         hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
 	UINT16			*src16;
@@ -1294,7 +1295,8 @@ void prism2sta_inf_tallies(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
 * Call context:
 *	interrupt
 ----------------------------------------------------------------*/
-void prism2sta_inf_scanresults(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
+static void prism2sta_inf_scanresults(wlandevice_t *wlandev,
+				      hfa384x_InfFrame_t *inf)
 {
 
         hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
@@ -1355,8 +1357,8 @@ void prism2sta_inf_scanresults(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
 * Call context:
 *	interrupt
 ----------------------------------------------------------------*/
-void prism2sta_inf_hostscanresults(wlandevice_t *wlandev, 
-				   hfa384x_InfFrame_t *inf)
+static void prism2sta_inf_hostscanresults(wlandevice_t *wlandev, 
+					  hfa384x_InfFrame_t *inf)
 {
         hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
 	int			nbss;
@@ -1401,7 +1403,8 @@ void prism2sta_inf_hostscanresults(wlandevice_t *wlandev,
 * Call context:
 *	interrupt
 ----------------------------------------------------------------*/
-void prism2sta_inf_chinforesults(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
+static void prism2sta_inf_chinforesults(wlandevice_t *wlandev,
+					hfa384x_InfFrame_t *inf)
 {
         hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
 	unsigned int		i, n;
@@ -1438,9 +1441,9 @@ void prism2sta_inf_chinforesults(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
 	return;
 }
 
-void prism2sta_processing_defer(void *data)
+void prism2sta_processing_defer(struct work_struct *data)
 {
-	hfa384x_t		*hw = (hfa384x_t *) data;
+	hfa384x_t		*hw = container_of(data, struct hfa384x, link_bh);
 	wlandevice_t            *wlandev = hw->wlandev;
 	hfa384x_bytestr32_t ssid;
 	int			result;
@@ -1540,7 +1543,7 @@ void prism2sta_processing_defer(void *data)
 				WLAN_MACMODE_IBSS_STA : WLAN_MACMODE_ESS_STA;
 
 			/* Get the ball rolling on the comms quality stuff */
-			prism2sta_commsqual_defer(hw);
+			prism2sta_commsqual_defer(&hw->commsqual_bh);
 		}
 		break;
 
@@ -1710,7 +1713,8 @@ void prism2sta_processing_defer(void *data)
 * Call context:
 *	interrupt
 ----------------------------------------------------------------*/
-void prism2sta_inf_linkstatus(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
+static void prism2sta_inf_linkstatus(wlandevice_t *wlandev,
+				     hfa384x_InfFrame_t *inf)
 {
         hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
 
@@ -1742,7 +1746,8 @@ void prism2sta_inf_linkstatus(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
 * Call context:
 *	interrupt
 ----------------------------------------------------------------*/
-void prism2sta_inf_assocstatus(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
+static void prism2sta_inf_assocstatus(wlandevice_t *wlandev,
+				      hfa384x_InfFrame_t *inf)
 {
         hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
 	hfa384x_AssocStatus_t	rec;
@@ -1805,7 +1810,8 @@ void prism2sta_inf_assocstatus(wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
 *	interrupt
 *
 ----------------------------------------------------------------*/
-void prism2sta_inf_authreq( wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
+static void prism2sta_inf_authreq(wlandevice_t *wlandev,
+				  hfa384x_InfFrame_t *inf)
 {
         hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
 	struct sk_buff *skb;
@@ -1823,8 +1829,8 @@ void prism2sta_inf_authreq( wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
 	DBFEXIT;
 }
 
-void prism2sta_inf_authreq_defer(wlandevice_t *wlandev, 
-				 hfa384x_InfFrame_t *inf)
+static void prism2sta_inf_authreq_defer(wlandevice_t *wlandev, 
+					hfa384x_InfFrame_t *inf)
 {
         hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
 	hfa384x_authenticateStation_data_t  rec;
@@ -1999,7 +2005,8 @@ void prism2sta_inf_authreq_defer(wlandevice_t *wlandev,
 * Call context:
 *	interrupt
 ----------------------------------------------------------------*/
-void prism2sta_inf_psusercnt( wlandevice_t *wlandev, hfa384x_InfFrame_t *inf)
+static void prism2sta_inf_psusercnt(wlandevice_t *wlandev,
+				    hfa384x_InfFrame_t *inf)
 {
         hfa384x_t               *hw = (hfa384x_t *)wlandev->priv;
 
@@ -2272,8 +2279,8 @@ static int prism2sta_resume_pci (struct pci_dev *pdev)
 	wlandev = (wlandevice_t *) pci_get_drvdata(pdev);
 
 	if (wlandev) {
-		prism2sta_ifstate(wlandev, P80211ENUM_ifstate_disable);
 		p80211_resume(wlandev);
+		prism2sta_ifstate(wlandev, P80211ENUM_ifstate_enable);
 	}
 		
         return 0;
@@ -2378,7 +2385,7 @@ prism2sta_proc_read(
 	hwtype |= BIT2;
 #endif
 
-#define PRISM2_CVS_ID "$Id: prism2sta.c 1767 2006-01-19 21:25:50Z solomon $"
+#define PRISM2_CVS_ID "$Id: prism2sta.c 1858 2008-03-24 18:49:31Z pizza $"
 
 	p += sprintf(p, "# %s version %s (%s) '%s'\n\n", 
 		     dev_info,
@@ -2413,9 +2420,9 @@ prism2sta_proc_read(
 }
 #endif
 
-void prism2sta_commsqual_defer(void *data)
+void prism2sta_commsqual_defer(struct work_struct *data)
 {
-        hfa384x_t               *hw = (hfa384x_t *) data;
+	hfa384x_t		*hw = container_of(data, struct hfa384x, commsqual_bh);
         wlandevice_t            *wlandev = hw->wlandev;
 	hfa384x_bytestr32_t ssid;
 	int result = 0;
@@ -2477,7 +2484,7 @@ void prism2sta_commsqual_defer(void *data)
 	
 
 	/* Reschedule timer */
-	mod_timer(&hw->commsqual_timer, jiffies + HZ);
+	mod_timer(&hw->commsqual_timer, round_jiffies(jiffies + HZ));
 
  done:
 	DBFEXIT;
