@@ -328,7 +328,7 @@ UINT32 p80211_text2itemdid(p80211meta_t *list, char *name )
 *	P80211DID_INVALID	if DID is an invalid DID
 *	!P80211DID_INVALID	if DID is a valid DID
 ----------------------------------------------------------------*/
-INT p80211_isvalid_did( catlistitem_t *catlist, UINT32 did )
+UINT32 p80211_isvalid_did( catlistitem_t *catlist, UINT32 did )
 {
 	int		result;
 
@@ -351,7 +351,7 @@ INT p80211_isvalid_did( catlistitem_t *catlist, UINT32 did )
 *	P80211DID_INVALID	if DID is an invalid DID
 *	!P80211DID_INVALID	if DID is a valid DID
 ----------------------------------------------------------------*/
-INT p80211_isvalid_catdid( catlistitem_t *catlist, UINT32 did )
+UINT32 p80211_isvalid_catdid( catlistitem_t *catlist, UINT32 did )
 {
 	int		result;
 	int		cat;
@@ -383,7 +383,7 @@ INT p80211_isvalid_catdid( catlistitem_t *catlist, UINT32 did )
 *	P80211DID_INVALID	if DID is an invalid DID
 *	!P80211DID_INVALID	if DID is a valid DID
 ----------------------------------------------------------------*/
-INT p80211_isvalid_grpdid( catlistitem_t *catlist, UINT32 did )
+UINT32 p80211_isvalid_grpdid( catlistitem_t *catlist, UINT32 did )
 {
 	int		result;
 	int		cat;
@@ -418,7 +418,7 @@ INT p80211_isvalid_grpdid( catlistitem_t *catlist, UINT32 did )
 *	P80211DID_INVALID	if DID is an invalid DID
 *	!P80211DID_INVALID	if DID is a valid DID
 ----------------------------------------------------------------*/
-INT p80211_isvalid_itemdid( catlistitem_t *catlist, UINT32 did )
+UINT32 p80211_isvalid_itemdid( catlistitem_t *catlist, UINT32 did )
 {
 	int		result;
 	int		cat;
@@ -614,8 +614,6 @@ int p80211item_gettype(p80211meta_t *meta)
 		type = P80211_TYPE_OCTETSTR;
 	else if (meta->totextptr == p80211_totext_displaystr)
 		type = P80211_TYPE_DISPLAYSTR;
-	else if (meta->totextptr == p80211_totext_boundedint)
-		type = P80211_TYPE_BOUNDEDINT;
 	else if (meta->totextptr == p80211_totext_int)
 		type = P80211_TYPE_INT;
 	else if (meta->totextptr == p80211_totext_enumint)
@@ -696,7 +694,6 @@ UINT32 p80211item_maxdatalen( catlistitem_t *metalist, UINT32 did )
 				break;
 
 			case P80211_TYPE_INT:
-			case P80211_TYPE_BOUNDEDINT:
 			case P80211_TYPE_ENUMINT:
 			case P80211_TYPE_BITARRAY:
 				/* all int types are 4 bytes */
